import { __assign, __read, __spread } from "tslib";
import * as React from "react";
import { useContext } from "react";
import { MotionConfigContext } from "../context/MotionConfigContext";
/**
 * Load features via renderless components based on the provided MotionProps
 */
export function useFeatures(defaultFeatures, isStatic, visualElement, props) {
    var plugins = useContext(MotionConfigContext);
    // If this is a static component, or we're rendering on the server, we don't load
    // any feature components
    if (isStatic || typeof window === "undefined")
        return null;
    var allFeatures = __spread(defaultFeatures, plugins.features);
    var numFeatures = allFeatures.length;
    var features = [];
    // Decide which features we should render and add them to the returned array
    for (var i = 0; i < numFeatures; i++) {
        var _a = allFeatures[i], shouldRender = _a.shouldRender, key = _a.key, getComponent = _a.getComponent;
        if (shouldRender(props)) {
            var Component = getComponent(props);
            Component &&
                features.push(React.createElement(Component, __assign({ key: key }, props, { visualElement: visualElement })));
        }
    }
    return features;
}
//# sourceMappingURL=use-features.js.map